@@ -309,6 +309,8 @@ static void copyLogInfo(struct logInfo *to, struct logInfo *from)
 	to->first = strdup(from->first);
     if (from->last)
 	to->last = strdup(from->last);
+    if (from->preremove)
+	to->preremove = strdup(from->preremove);
     if (from->logAddress)
 	to->logAddress = strdup(from->logAddress);
     if (from->extension)
@@ -343,6 +345,7 @@ static void freeLogInfo(struct logInfo *log)
 	free(log->post);
 	free(log->first);
 	free(log->last);
+	free(log->preremove);
 	free(log->logAddress);
 	free(log->extension);
 	free(log->compress_prog);
@@ -381,7 +384,8 @@ static void freeTailLogs(int num)
 	message(MESS_DEBUG, "removing last %d log configs\n", num);
 
 	while (num--)
-		removeLogInfo(*(logs.tqh_last));
+		removeLogInfo(TAILQ_LAST(&logs, logInfoHead));
+
 }
 
 static int readConfigPath(const char *path, struct logInfo *defConfig)
@@ -521,6 +525,7 @@ int readAllConfigPaths(const char **paths)
 		.post = NULL,
 		.first = NULL,
 		.last = NULL,
+		.preremove = NULL,
 		.logAddress = NULL,
 		.extension = NULL,
 		.compress_prog = NULL,
@@ -624,12 +629,14 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 	if ((flags = fcntl(fd, F_GETFD)) == -1) {
 		message(MESS_ERROR, "Could not retrieve flags from file %s\n",
 				configFile);
+		close(fd);
 		return 1;
 	}
 	flags |= FD_CLOEXEC;
 	if (fcntl(fd, F_SETFD, flags) == -1) {
 		message(MESS_ERROR, "Could not set flags on file %s\n",
 				configFile);
+		close(fd);
 		return 1;
 	}
 	/* We don't want anybody to change the file while we parse it,
@@ -654,6 +661,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
     
 	if (!(pw = getpwuid(getuid()))) {
 		message(MESS_ERROR, "Logrotate UID is not in passwd file.\n");
+		close(fd);
 		return 1;
 	}
 
@@ -915,7 +923,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 					if ((key = isolateValue(configFile, lineNum, opt, &start,
 							&buf, length)) != NULL) {
 						int l = strlen(key) - 1;
-						if (key[l] == 'k') {
+						if (key[l] == 'k' || key[l] == 'K') {
 							key[l] = '\0';
 							multiplier = 1024;
 						} else if (key[l] == 'M') {
@@ -1084,6 +1092,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 					scriptStart = start;
 					scriptDest = &newlog->last;
 					state = STATE_LOAD_SCRIPT;
+				} else if (!strcmp(key, "preremove")) {
+					freeLogItem (preremove);
+					scriptStart = start;
+					scriptDest = &newlog->preremove;
+					state = STATE_LOAD_SCRIPT;
 				} else if (!strcmp(key, "tabooext")) {
 					if (newlog != defConfig) {
 						message(MESS_ERROR,
@@ -1123,8 +1136,8 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
 							endtag = chptr;
 							if (*endtag == ',')
-								start++;
-							while (isspace(*endtag) && *endtag)
+								endtag++;
+							while (*endtag && isspace(*endtag))
 								endtag++;
 						}
 					}
@@ -1439,12 +1452,8 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
 				newlog->pattern = key;
 
-// 				if (!logerror)
-// 				message(MESS_DEBUG, "reading config info for %s\n", start);
-
 				free(argv);
 
-// 				start = endtag + 1;
 			} else if (*start == '}') {
 				if (newlog == defConfig) {
 					message(MESS_ERROR, "%s:%d unexpected }\n", configFile,
@@ -1463,7 +1472,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 				free(globerr_msg);
 				globerr_msg = NULL;
 				if (!(newlog->flags & LOG_FLAG_MISSINGOK))
-					return 1;
+					goto error;
 				}
 
 				if (newlog->oldDir) {
@@ -1588,13 +1597,30 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 				if (
 					(strcmp(key, "postrotate") == 0) ||
 					(strcmp(key, "prerotate") == 0) ||
-					(strcmp(key, "firstrotate") == 0) ||
-					(strcmp(key, "lastrotate") == 0)
+					(strcmp(key, "firstaction") == 0) ||
+					(strcmp(key, "lastaction") == 0) ||
+					(strcmp(key, "preremove") == 0)
 					) {
 					state = STATE_LOAD_SCRIPT | STATE_SKIP_CONFIG;
 				}
 				else {
-					state = STATE_SKIP_LINE | STATE_SKIP_CONFIG;
+					/* isolateWord moves the "start" pointer.
+					 * If we have a line like
+					 *    rotate 5 
+					 * after isolateWord "start" points to "5" and it
+					 * is OK to skip the line, but if we have a line
+					 * like the following
+					 *    nocompress
+					 * after isolateWord "start" points to "\n". In
+					 * this case if we skip a line, we skip the next 
+					 * line, not the current "nocompress" one, 
+					 * because in the for cycle the "start"
+					 * pointer is increased by one and, after this, 
+					 * "start" points to the beginning of the next line.
+					*/
+					if (*start != '\n') {
+						state = STATE_SKIP_LINE | STATE_SKIP_CONFIG;
+					}
 				}
 				free(key);
 				key = NULL;
@@ -1613,7 +1639,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
  
     if (scriptStart) {
 	message(MESS_ERROR,
-		"%s:prerotate or postrotate without endscript\n",
+		"%s:prerotate, postrotate or preremove without endscript\n",
 		configFile);
 	goto error;
     }
