@@ -1,5 +1,8 @@
 #include <sys/queue.h>
+/* alloca() is defined in stdlib.h in NetBSD */
+#ifndef __NetBSD__
 #include <alloca.h>
+#endif
 #include <limits.h>
 #include <ctype.h>
 #include <dirent.h>
@@ -31,9 +34,13 @@ int selinux_enforce = 0;
 
 #ifdef WITH_ACL
 #include "sys/acl.h"
-static acl_t prev_acl = NULL;
+#define acl_type acl_t
+#else
+#define acl_type void *
 #endif
 
+static acl_type prev_acl = NULL;
+
 #include "basenames.h"
 #include "log.h"
 #include "logrotate.h"
@@ -110,6 +117,29 @@ int switch_user_back() {
 	return switch_user(save_euid, save_egid);
 }
 
+int switch_user_permanently(const struct logInfo *log) {
+	gid_t group = getegid();
+	uid_t user = geteuid();
+	if (!(log->flags & LOG_FLAG_SU)) {
+		return 0;
+	}
+	if (getuid() == user && getgid() == group)
+		return 0;
+	// switch to full root first
+	if (setgid(getgid()) || setuid(getuid())) {
+		message(MESS_ERROR, "error getting rid of euid != uid\n");
+		return 1;
+	}
+	message(MESS_DEBUG, "switching uid to %d and gid to %d\n",
+		user, group);
+	if (setgid(group) || setuid(user)) {
+		message(MESS_ERROR, "error switching euid to %d and egid to %d: %s\n",
+			user, group, strerror(errno));
+		return 1;
+	}
+	return 0;
+}
+
 static void unescape(char *arg)
 {
 	char *p = arg;
@@ -239,7 +269,7 @@ static struct logState *findState(const char *fn)
 	return p;
 }
 
-static int runScript(char *logfn, char *script)
+static int runScript(struct logInfo *log, char *logfn, char *script)
 {
     int rc;
 
@@ -250,6 +280,11 @@ static int runScript(char *logfn, char *script)
     }
 
 	if (!fork()) {
+		if (log->flags & LOG_FLAG_SU) {
+			if (switch_user_back() != 0) {
+				exit(1);
+			}
+		}
 		execl("/bin/sh", "sh", "-c", script, "logrotate_script", logfn, NULL);
 		exit(1);
 	}
@@ -258,9 +293,10 @@ static int runScript(char *logfn, char *script)
     return rc;
 }
 
-int createOutputFile(char *fileName, int flags, struct stat *sb)
+int createOutputFile(char *fileName, int flags, struct stat *sb, acl_type acl, int force_mode)
 {
     int fd;
+	struct stat sb_create;
 
 	fd = open(fileName, (flags | O_EXCL | O_NOFOLLOW),
 		(S_IRUSR | S_IWUSR) & sb->st_mode);
@@ -276,18 +312,42 @@ int createOutputFile(char *fileName, int flags, struct stat *sb)
 	close(fd);
 	return -1;
     }
-    if (fchown(fd, sb->st_uid, sb->st_gid)) {
-	message(MESS_ERROR, "error setting owner of %s: %s\n",
-		fileName, strerror(errno));
+
+	if (fstat(fd, &sb_create)) {
+		message(MESS_ERROR, "fstat of %s failed: %s\n", fileName,
+			strerror(errno));
+		close(fd);
+		return -1;
+	}
+ 
+    if ((sb_create.st_uid != sb->st_uid || sb_create.st_gid != sb->st_gid) && 
+		fchown(fd, sb->st_uid, sb->st_gid)) {
+	message(MESS_ERROR, "error setting owner of %s to uid %d and gid %d: %s\n",
+		fileName, sb->st_uid, sb->st_gid, strerror(errno));
 	close(fd);
 	return -1;
     }
+
     if (fchmod(fd, sb->st_mode)) {
 	message(MESS_ERROR, "error setting mode of %s: %s\n",
 		fileName, strerror(errno));
 	close(fd);
 	return -1;
     }
+
+#ifdef WITH_ACL
+	if (!force_mode && acl) {
+		if (acl_set_fd(fd, acl) == -1) {
+			if (errno != ENOTSUP) {
+				message(MESS_ERROR, "setting ACL for %s: %s\n",
+				fileName, strerror(errno));
+				close(fd);
+				return 1;
+			}
+		}
+	}
+#endif
+
     return fd;
 }
 
@@ -301,6 +361,19 @@ static int shred_file(int fd, char *filename, struct logInfo *log)
 	int id = 0;
 	int status;
 
+	if (log->preremove) {
+	    message(MESS_DEBUG, "running preremove script\n");
+	    if (runScript(log, filename, log->preremove)) {
+		    message(MESS_ERROR,
+			    "error running preremove script "
+			    "for %s of '%s'. Not removing this file.\n",
+			    filename, log->pattern);
+		    /* What ever was supposed to happen did not happen,
+		     * therefore do not unlink the file yet.  */
+		    return 1;
+	    }
+	}
+
 	if (!(log->flags & LOG_FLAG_SHRED)) {
 		return unlink(filename);
 	}
@@ -328,6 +401,10 @@ static int shred_file(int fd, char *filename, struct logInfo *log)
 		dup2(fd, 1);
 		close(fd);
 
+		if (switch_user_permanently(log) != 0) {
+			exit(1);
+		}
+
 		execvp(fullCommand[0], (void *) fullCommand);
 		exit(1);
 	}
@@ -349,7 +426,7 @@ static int removeLogFile(char *name, struct logInfo *log)
 	int fd;
 	message(MESS_DEBUG, "removing old log %s\n", name);
 
-	if ((fd = open(name, O_RDWR)) < 0) {
+	if ((fd = open(name, O_RDWR | O_NOFOLLOW)) < 0) {
 		message(MESS_ERROR, "error opening %s: %s\n",
 			name, strerror(errno));
 		return 1;
@@ -390,44 +467,34 @@ static int compressLogFile(char *name, struct logInfo *log, struct stat *sb)
     compressedName = alloca(strlen(name) + strlen(log->compress_ext) + 2);
     sprintf(compressedName, "%s%s", name, log->compress_ext);
 
-    if ((inFile = open(name, O_RDWR)) < 0) {
+    if ((inFile = open(name, O_RDWR | O_NOFOLLOW)) < 0) {
 	message(MESS_ERROR, "unable to open %s for compression\n", name);
 	return 1;
     }
 
-    outFile =
-	createOutputFile(compressedName, O_RDWR | O_CREAT, sb);
-    if (outFile < 0) {
-	close(inFile);
-	return 1;
-    }
-
 #ifdef WITH_ACL
 	if ((prev_acl = acl_get_fd(inFile)) == NULL) {
 		if (errno != ENOTSUP) {
 			message(MESS_ERROR, "getting file ACL %s: %s\n",
 				name, strerror(errno));
 			close(inFile);
-			close(outFile);
 			return 1;
 		}
 	}
+#endif
+
+    outFile =
+	createOutputFile(compressedName, O_RDWR | O_CREAT, sb, prev_acl, 0);
+#ifdef WITH_ACL
 	if (prev_acl) {
-		if (acl_set_fd(outFile, prev_acl) == -1) {
-			if (errno != ENOTSUP) {
-				message(MESS_ERROR, "setting ACL for %s: %s\n",
-				compressedName, strerror(errno));
-				acl_free(prev_acl);
-				prev_acl = NULL;
-				close(inFile);
-				close(outFile);
-				return 1;
-			}
-		}
 		acl_free(prev_acl);
 		prev_acl = NULL;
 	}
-#endif /* WITH_ACL */
+#endif
+    if (outFile < 0) {
+	close(inFile);
+	return 1;
+    }
 
     if (!fork()) {
 	dup2(inFile, 0);
@@ -435,6 +502,10 @@ static int compressLogFile(char *name, struct logInfo *log, struct stat *sb)
 	dup2(outFile, 1);
 	close(outFile);
 
+	if (switch_user_permanently(log) != 0) {
+		exit(1);
+	}
+
 	execvp(fullCommand[0], (void *) fullCommand);
 	exit(1);
     }
@@ -445,6 +516,7 @@ static int compressLogFile(char *name, struct logInfo *log, struct stat *sb)
 
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
 	message(MESS_ERROR, "failed to compress log %s\n", name);
+	close(inFile);
 	return 1;
     }
 
@@ -460,7 +532,7 @@ static int compressLogFile(char *name, struct logInfo *log, struct stat *sb)
     return 0;
 }
 
-static int mailLog(char *logFile, char *mailCommand,
+static int mailLog(struct logInfo *log, char *logFile, char *mailCommand,
 		   char *uncompressCommand, char *address, char *subject)
 {
     int mailInput;
@@ -470,7 +542,7 @@ static int mailLog(char *logFile, char *mailCommand,
     char *mailArgv[] = { mailCommand, "-s", subject, address, NULL };
     int rc = 0;
 
-    if ((mailInput = open(logFile, O_RDONLY)) < 0) {
+    if ((mailInput = open(logFile, O_RDONLY | O_NOFOLLOW)) < 0) {
 	message(MESS_ERROR, "failed to open %s for mailing: %s\n", logFile,
 		strerror(errno));
 	return 1;
@@ -480,6 +552,7 @@ static int mailLog(char *logFile, char *mailCommand,
 		if (pipe(uncompressPipe) < 0) {
 			message(MESS_ERROR, "error opening pipe for uncompress: %s",
 					strerror(errno));
+			close(mailInput);
 			return 1;
 		}
 		if (!(uncompressChild = fork())) {
@@ -490,6 +563,10 @@ static int mailLog(char *logFile, char *mailCommand,
 			close(uncompressPipe[0]);
 			close(uncompressPipe[1]);
 
+			if (switch_user_permanently(log) != 0) {
+				exit(1);
+			}
+
 			execlp(uncompressCommand, uncompressCommand, NULL);
 			exit(1);
 		}
@@ -504,6 +581,13 @@ static int mailLog(char *logFile, char *mailCommand,
 	close(mailInput);
 	close(1);
 
+	// mail command runs as root
+	if (log->flags & LOG_FLAG_SU) {
+		if (switch_user_back() != 0) {
+			exit(1);
+		}
+	}
+
 	execvp(mailArgv[0], mailArgv);
 	exit(1);
     }
@@ -539,12 +623,12 @@ static int mailLogWrapper(char *mailFilename, char *mailCommand,
     if ((log->flags & LOG_FLAG_COMPRESS) &&
 	!((log->flags & LOG_FLAG_DELAYCOMPRESS) &&
 	  (log->flags & LOG_FLAG_MAILFIRST))) {
-	if (mailLog(mailFilename, mailCommand,
+	if (mailLog(log, mailFilename, mailCommand,
 		    log->uncompress_prog, log->logAddress,
 		    log->files[logNum]))
 	    return 1;
     } else {
-	if (mailLog(mailFilename, mailCommand, NULL,
+	if (mailLog(log, mailFilename, mailCommand, NULL,
 		    log->logAddress, mailFilename))
 	    return 1;
     }
@@ -561,7 +645,7 @@ static int copyTruncate(char *currLog, char *saveLog, struct stat *sb,
     message(MESS_DEBUG, "copying %s to %s\n", currLog, saveLog);
 
     if (!debug) {
-	if ((fdcurr = open(currLog, (flags & LOG_FLAG_COPY) ? O_RDONLY : O_RDWR)) < 0) {
+	if ((fdcurr = open(currLog, ((flags & LOG_FLAG_COPY) ? O_RDONLY : O_RDWR) | O_NOFOLLOW)) < 0) {
 	    message(MESS_ERROR, "error opening %s: %s\n", currLog,
 		    strerror(errno));
 	    return 1;
@@ -576,6 +660,7 @@ static int copyTruncate(char *currLog, char *saveLog, struct stat *sb,
 			    strerror(errno));
 		    if (selinux_enforce) {
 				freecon(oldContext);
+				close(fdcurr);
 				return 1;
 		    }
 		}
@@ -585,6 +670,7 @@ static int copyTruncate(char *currLog, char *saveLog, struct stat *sb,
 			    saveLog, oldContext, strerror(errno));
 			if (selinux_enforce) {
 				freecon(oldContext);
+				close(fdcurr);
 				return 1;
 		    }
 		}
@@ -612,7 +698,7 @@ static int copyTruncate(char *currLog, char *saveLog, struct stat *sb,
 	}
 #endif /* WITH_ACL */
 	fdsave =
-	    createOutputFile(saveLog, O_WRONLY | O_CREAT, sb);
+	    createOutputFile(saveLog, O_WRONLY | O_CREAT, sb, prev_acl, 0);
 #ifdef WITH_SELINUX
 	if (selinux_enabled) {
 	    setfscreatecon_raw(prev_context);
@@ -620,31 +706,16 @@ static int copyTruncate(char *currLog, char *saveLog, struct stat *sb,
 		prev_context = NULL;
 	}
 #endif
-	if (fdsave < 0) {
-	    close(fdcurr);
-#ifdef WITH_ACL
-		if (prev_acl)
-			acl_free(prev_acl);
-#endif /* WITH_ACL */
-	    return 1;
-	}
 #ifdef WITH_ACL
 	if (prev_acl) {
-		if ((fdsave, prev_acl) == -1) {
-			if (errno != ENOTSUP) {
-				message(MESS_ERROR, "setting ACL for %s: %s\n",
-				saveLog, strerror(errno));
-				acl_free(prev_acl);
-				prev_acl = NULL;
-				close(fdsave);
-				close(fdcurr);
-				return 1;
-			}
-		}
 		acl_free(prev_acl);
 		prev_acl = NULL;
 	}
-#endif /* WITH_ACL */
+#endif
+	if (fdsave < 0) {
+	    close(fdcurr);
+	    return 1;
+	}
 
 	while ((cnt = read(fdcurr, buf, sizeof(buf))) > 0) {
 	    if (write(fdsave, buf, cnt) != cnt) {
@@ -718,12 +789,12 @@ int findNeedRotating(struct logInfo *log, int logNum)
 								" should be used for rotation.\n"
 								,log->files[logNum]);
 			free(ld);
-			return 0;
+			return 1;
 		}
 		free(ld);
 	}
 
-    if (stat(log->files[logNum], &sb)) {
+    if (lstat(log->files[logNum], &sb)) {
 	if ((log->flags & LOG_FLAG_MISSINGOK) && (errno == ENOENT)) {
 	    message(MESS_DEBUG, "  log %s does not exist -- skipping\n",
 		    log->files[logNum]);
@@ -738,6 +809,13 @@ int findNeedRotating(struct logInfo *log, int logNum)
     state->doRotate = 0;
     state->sb = sb;
 
+	if ((sb.st_mode & S_IFMT) == S_IFLNK) {
+	    message(MESS_DEBUG, "  log %s is symbolic link. Rotation of symbolic"
+			" links is not allowed to avoid security issues -- skipping.\n",
+		    log->files[logNum]);
+		return 0;
+	}
+
     if (log->criterium == ROT_SIZE) {
 	state->doRotate = (sb.st_size >= log->threshhold);
     } else if (log->criterium == ROT_FORCE) {
@@ -960,6 +1038,42 @@ int prerotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 	message(MESS_DEBUG, "dateext suffix '%s'\n", dext_str);
 	message(MESS_DEBUG, "glob pattern '%s'\n", dext_pattern);
 
+#ifdef WITH_SELINUX
+	if (selinux_enabled) {
+	    security_context_t oldContext = NULL;
+	    if (getfilecon_raw(log->files[logNum], &oldContext) > 0) {
+			if (getfscreatecon_raw(&prev_context) < 0) {
+				message(MESS_ERROR,
+					"getting default context: %s\n",
+					strerror(errno));
+				if (selinux_enforce) {
+					freecon(oldContext);
+					return 1;
+				}
+			}
+			if (setfscreatecon_raw(oldContext) < 0) {
+				message(MESS_ERROR,
+					"setting file context %s to %s: %s\n",
+					log->files[logNum], oldContext,
+					strerror(errno));
+				if (selinux_enforce) {
+					freecon(oldContext);
+					return 1;
+				}
+			}
+			freecon(oldContext);
+	    } else {
+		if (errno != ENOENT && errno != ENOTSUP) {
+			message(MESS_ERROR, "getting file context %s: %s\n",
+				log->files[logNum], strerror(errno));
+			if (selinux_enforce) {
+				return 1;
+			}
+		}
+	    }
+	}
+#endif
+
     /* First compress the previous log when necessary */
     if (log->flags & LOG_FLAG_COMPRESS &&
 	log->flags & LOG_FLAG_DELAYCOMPRESS) {
@@ -1070,7 +1184,8 @@ int prerotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 	}
 	/* firstRotated is most recently created/compressed rotated log */
 	sprintf(rotNames->firstRotated, "%s/%s%s%s%s",
-		rotNames->dirName, rotNames->baseName, dext_str, fileext, compext);
+		rotNames->dirName, rotNames->baseName, dext_str, fileext,
+		(log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext);
 	globfree(&globResult);
 	free(glob_pattern);
     } else {
@@ -1105,42 +1220,6 @@ int prerotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 		rotNames->baseName, logStart, fileext,
 		(log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext);
 
-#ifdef WITH_SELINUX
-	if (selinux_enabled) {
-	    security_context_t oldContext = NULL;
-	    if (getfilecon_raw(log->files[logNum], &oldContext) > 0) {
-			if (getfscreatecon_raw(&prev_context) < 0) {
-				message(MESS_ERROR,
-					"getting default context: %s\n",
-					strerror(errno));
-				if (selinux_enforce) {
-					freecon(oldContext);
-					return 1;
-				}
-			}
-			if (setfscreatecon_raw(oldContext) < 0) {
-				message(MESS_ERROR,
-					"setting file context %s to %s: %s\n",
-					log->files[logNum], oldContext,
-					strerror(errno));
-				if (selinux_enforce) {
-					freecon(oldContext);
-					return 1;
-				}
-			}
-			freecon(oldContext);
-	    } else {
-		if (errno != ENOENT && errno != ENOTSUP) {
-			message(MESS_ERROR, "getting file context %s: %s\n",
-				log->files[logNum], strerror(errno));
-			if (selinux_enforce) {
-				return 1;
-			}
-		}
-	    }
-	}
-#endif
-
 	for (i = rotateCount + logStart - 1; (i >= 0) && !hasErrors; i--) {
 		free(newName);
 		newName = oldName;
@@ -1226,7 +1305,7 @@ int rotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 			security_context_t oldContext = NULL;
 			int fdcurr = -1;
 
-			if ((fdcurr = open(log->files[logNum], O_RDWR)) < 0) {
+			if ((fdcurr = open(log->files[logNum], O_RDWR | O_NOFOLLOW)) < 0) {
 				message(MESS_ERROR, "error opening %s: %s\n",
 						log->files[logNum],
 					strerror(errno));
@@ -1323,10 +1402,13 @@ int rotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 	    else
 		sb.st_gid = log->createGid;
 
+	    int have_create_mode = 0;
 	    if (log->createMode == NO_MODE)
 		sb.st_mode = state->sb.st_mode & 0777;
-	    else
+	    else {
 		sb.st_mode = log->createMode;
+		have_create_mode = 1;
+	    }
 
 	    message(MESS_DEBUG, "creating new %s mode = 0%o uid = %d "
 		    "gid = %d\n", log->files[logNum], (unsigned int) sb.st_mode,
@@ -1335,23 +1417,16 @@ int rotateSingleLog(struct logInfo *log, int logNum, struct logState *state,
 	    if (!debug) {
 			if (!hasErrors) {
 			fd = createOutputFile(log->files[logNum], O_CREAT | O_RDWR,
-						  &sb);
+						  &sb, prev_acl, have_create_mode);
+#ifdef WITH_ACL
+			if (prev_acl) {
+				acl_free(prev_acl);
+				prev_acl = NULL;
+			}
+#endif
 			if (fd < 0)
 				hasErrors = 1;
 			else {
-#ifdef WITH_ACL
-				if (prev_acl) {
-					if (acl_set_fd(fd, prev_acl) == -1) {
-						if (errno != ENOTSUP) {
-							message(MESS_ERROR, "setting ACL for %s: %s\n",
-							log->files[logNum], strerror(errno));
-							hasErrors = 1;
-						}
-					}
-					acl_free(prev_acl);
-					prev_acl = NULL;
-				}
-#endif /* WITH_ACL */
 				close(fd);
 			}
 			}
@@ -1473,7 +1548,7 @@ int rotateLogSet(struct logInfo *log, int force)
 	message(MESS_DEBUG, "only log files >= %llu bytes are rotated, ",	log->minsize);
 
     if (log->maxsize) 
-	message(MESS_DEBUG, "log files >= %llu are rotated earlier, ",	log->minsize);
+	message(MESS_DEBUG, "log files >= %llu are rotated earlier, ",	log->maxsize);
 
     if (log->logAddress) {
 	message(MESS_DEBUG, "old logs mailed to %s\n", log->logAddress);
@@ -1502,7 +1577,7 @@ int rotateLogSet(struct logInfo *log, int force)
 		    "since no logs will be rotated\n");
 	} else {
 	    message(MESS_DEBUG, "running first action script\n");
-	    if (runScript(log->pattern, log->first)) {
+	    if (runScript(log, log->pattern, log->first)) {
 		message(MESS_ERROR, "error running first action script "
 			"for %s\n", log->pattern);
 		hasErrors = 1;
@@ -1537,15 +1612,16 @@ int rotateLogSet(struct logInfo *log, int force)
 	    hasErrors |= logHasErrors[i];
 	}
 
+	/* (nemam chyby nebo shared) a (nemam chyby nebo nonshared) */
 	if (log->pre
-	    && (! ( (logHasErrors[j] && !(log->flags & LOG_FLAG_SHAREDSCRIPTS))
-		   || (hasErrors && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) ) )) {
+	    && (!logHasErrors[j] || log->flags & LOG_FLAG_SHAREDSCRIPTS) &&
+		   ((!hasErrors && state[j]->doRotate) || (!(log->flags & LOG_FLAG_SHAREDSCRIPTS) && state[j]->doRotate) ) ) {
 	    if (!numRotated) {
 		message(MESS_DEBUG, "not running prerotate script, "
 			"since no logs will be rotated\n");
 	    } else {
 		message(MESS_DEBUG, "running prerotate script\n");
-		if (runScript(log->flags & LOG_FLAG_SHAREDSCRIPTS ? log->pattern : log->files[j], log->pre)) {
+		if (runScript(log, log->flags & LOG_FLAG_SHAREDSCRIPTS ? log->pattern : log->files[j], log->pre)) {
 		    if (log->flags & LOG_FLAG_SHAREDSCRIPTS)
 			message(MESS_ERROR,
 				"error running shared prerotate script "
@@ -1573,14 +1649,14 @@ int rotateLogSet(struct logInfo *log, int force)
 	}
 
 	if (log->post
-	    && (! ( (logHasErrors[j] && !(log->flags & LOG_FLAG_SHAREDSCRIPTS))
-		   || (hasErrors && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) ) )) {
+	    && (!logHasErrors[j] || log->flags & LOG_FLAG_SHAREDSCRIPTS) &&
+		   ((!hasErrors && state[j]->doRotate) || (!(log->flags & LOG_FLAG_SHAREDSCRIPTS) && state[j]->doRotate) ) ) {
 	    if (!numRotated) {
 		message(MESS_DEBUG, "not running postrotate script, "
 			"since no logs were rotated\n");
 	    } else {
 		message(MESS_DEBUG, "running postrotate script\n");
-		if (runScript(log->flags & LOG_FLAG_SHAREDSCRIPTS ? log->pattern : log->files[j], log->post)) {
+		if (runScript(log, log->flags & LOG_FLAG_SHAREDSCRIPTS ? log->pattern : log->files[j], log->post)) {
 		    if (log->flags & LOG_FLAG_SHAREDSCRIPTS)
 			message(MESS_ERROR,
 				"error running shared postrotate script "
@@ -1626,7 +1702,7 @@ int rotateLogSet(struct logInfo *log, int force)
 		    "since no logs will be rotated\n");
 	} else {
 	    message(MESS_DEBUG, "running last action script\n");
-	    if (runScript(log->pattern, log->last)) {
+	    if (runScript(log, log->pattern, log->last)) {
 		message(MESS_ERROR, "error running last action script "
 			"for %s\n", log->pattern);
 		hasErrors = 1;
@@ -1851,6 +1927,7 @@ int main(int argc, const char **argv)
 	 "Path of state file",
 	 "statefile"},
 	{"verbose", 'v', 0, 0, 'v', "Display messages during rotation"},
+	{"version", '\0', POPT_ARG_NONE, NULL, 'V', "Display version information"},
 	POPT_AUTOHELP {0, 0, 0, 0, 0}
     };
 
@@ -1869,6 +1946,10 @@ int main(int argc, const char **argv)
 	case 'v':
 	    logSetLevel(MESS_DEBUG);
 	    break;
+	case 'V':
+	    fprintf(stderr, "logrotate %s\n", VERSION);
+	    poptFreeContext(optCon);
+	    exit(0);
 	}
     }
 
